Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Imports System.Threading.Tasks

Public Class AsyncCommand(Of TResult)
	Inherits AsyncCommandBase
	Implements INotifyPropertyChanged
	Private ReadOnly _command As Func(Of Task(Of TResult))
	Private _execution As NotifyTaskCompletion(Of TResult)

	Public Sub New(ByVal command As Func(Of Task(Of TResult)))
		_command = command
	End Sub

	Public Overrides Function CanExecute(ByVal parameter As Object) As Boolean
		Return True
	End Function

	Public Overrides Function ExecuteAsync(ByVal parameter As Object) As Task
		Execution = New NotifyTaskCompletion(Of TResult)(_command())
		Return Execution.TaskCompletion
	End Function

	Public Property Execution() As NotifyTaskCompletion(Of TResult)
		Get
			Return _execution
		End Get
		Private Set(ByVal value As NotifyTaskCompletion(Of TResult))
			_execution = value
			OnPropertyChanged()
		End Set
	End Property

	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub OnPropertyChanged(<CallerMemberName> Optional ByVal propertyName As String = Nothing)
        Dim handler As PropertyChangedEventHandler = PropertyChangedEvent
        If handler IsNot Nothing Then
            handler(Me, New PropertyChangedEventArgs(propertyName))
        End If
    End Sub
End Class

Public NotInheritable Class AsyncCommand
	Private Sub New()
	End Sub
	Public Shared Function Create(ByVal command As Func(Of Task)) As AsyncCommand(Of Object)
        Return New AsyncCommand(Of Object)(Async Function()
                                               Await command()
                                               Return Nothing
                                           End Function)
	End Function

	Public Shared Function Create(Of TResult)(ByVal command As Func(Of Task(Of TResult))) As AsyncCommand(Of TResult)
		Return New AsyncCommand(Of TResult)(command)
	End Function
End Class
