Imports System.ComponentModel
Imports System.Threading.Tasks
Public NotInheritable Class NotifyTaskCompletion(Of TResult)
	Implements INotifyPropertyChanged
	Public Sub New(ByVal task As Task(Of TResult))
		Me.Task = task
		TaskCompletion = WatchTaskAsync(task)
	End Sub
	Private async Function WatchTaskAsync(ByVal task As Task) As Task
		Try
            Await task
		Catch
		End Try
        Dim propertyChanged = PropertyChangedEvent
		If propertyChanged Is Nothing Then
			Return
		End If
		propertyChanged(Me, New PropertyChangedEventArgs("Status"))
		propertyChanged(Me, New PropertyChangedEventArgs("IsCompleted"))
		propertyChanged(Me, New PropertyChangedEventArgs("IsNotCompleted"))
		If task.IsCanceled Then
			propertyChanged(Me, New PropertyChangedEventArgs("IsCanceled"))
		ElseIf task.IsFaulted Then
			propertyChanged(Me, New PropertyChangedEventArgs("IsFaulted"))
			propertyChanged(Me, New PropertyChangedEventArgs("Exception"))
			propertyChanged(Me, New PropertyChangedEventArgs("InnerException"))
			propertyChanged(Me, New PropertyChangedEventArgs("ErrorMessage"))
		Else
			propertyChanged(Me, New PropertyChangedEventArgs("IsSuccessfullyCompleted"))
			propertyChanged(Me, New PropertyChangedEventArgs("Result"))
		End If
	End Function
	Private privateTask As Task(Of TResult)
	Public Property Task() As Task(Of TResult)
		Get
			Return privateTask
		End Get
		Private Set(ByVal value As Task(Of TResult))
			privateTask = value
		End Set
	End Property
	Private privateTaskCompletion As Task
	Public Property TaskCompletion() As Task
		Get
			Return privateTaskCompletion
		End Get
		Private Set(ByVal value As Task)
			privateTaskCompletion = value
		End Set
	End Property
	Public ReadOnly Property Result() As TResult
		Get
			Return If(Task.Status = TaskStatus.RanToCompletion, Task.Result, Nothing)
		End Get
	End Property
	Public ReadOnly Property Status() As TaskStatus
		Get
			Return Task.Status
		End Get
	End Property
	Public ReadOnly Property IsCompleted() As Boolean
		Get
			Return Task.IsCompleted
		End Get
	End Property
	Public ReadOnly Property IsNotCompleted() As Boolean
		Get
			Return Not Task.IsCompleted
		End Get
	End Property
	Public ReadOnly Property IsSuccessfullyCompleted() As Boolean
		Get
			Return Task.Status = TaskStatus.RanToCompletion
		End Get
	End Property
	Public ReadOnly Property IsCanceled() As Boolean
		Get
			Return Task.IsCanceled
		End Get
	End Property
	Public ReadOnly Property IsFaulted() As Boolean
		Get
			Return Task.IsFaulted
		End Get
	End Property
	Public ReadOnly Property Exception() As AggregateException
		Get
			Return Task.Exception
		End Get
	End Property
	Public ReadOnly Property InnerException() As Exception
		Get
			Return If(Exception Is Nothing, Nothing, Exception.InnerException)
		End Get
	End Property
	Public ReadOnly Property ErrorMessage() As String
		Get
			Return If(InnerException Is Nothing, Nothing, InnerException.Message)
		End Get
	End Property
	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class
