Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Imports System.Threading

Public NotInheritable Class CountUrlBytesViewModel
    Public Sub New(ByVal parent As MainWindowViewModel, ByVal url As String, ByVal command As IAsyncCommand)
        LoadingMessage = "Loading (" & url & ")..."
        Me.Command = command
        RemoveCommand = New DelegateCommand(Sub()
                                                parent.Operations.Remove(Me)
                                            End Sub)
    End Sub

    Private privateLoadingMessage As String
    Public Property LoadingMessage() As String
        Get
            Return privateLoadingMessage
        End Get
        Private Set(ByVal value As String)
            privateLoadingMessage = value
        End Set
    End Property

    Private privateCommand As IAsyncCommand
    Public Property Command() As IAsyncCommand
        Get
            Return privateCommand
        End Get
        Private Set(ByVal value As IAsyncCommand)
            privateCommand = value
        End Set
    End Property

    Private privateRemoveCommand As ICommand
    Public Property RemoveCommand() As ICommand
        Get
            Return privateRemoveCommand
        End Get
        Private Set(ByVal value As ICommand)
            privateRemoveCommand = value
        End Set
    End Property
End Class

Public NotInheritable Class MainWindowViewModel
    Implements INotifyPropertyChanged
    Public Sub New()
        Url = "http://www.example.com/"
        Operations = New ObservableCollection(Of CountUrlBytesViewModel)()
        CountUrlBytesCommand = New DelegateCommand(Sub()
                                                       Dim countBytes = AsyncCommand.Create(Function(token As CancellationToken) MyService.DownloadAndCountBytesAsync(Url, token))
                                                       countBytes.Execute(Nothing)
                                                       Operations.Add(New CountUrlBytesViewModel(Me, Url, countBytes))
                                                   End Sub
                                                )
    End Sub

    Private _url As String
    Public Property Url() As String
        Get
            Return _url
        End Get
        Set(ByVal value As String)
            _url = value
            OnPropertyChanged()
        End Set
    End Property

    Private privateOperations As ObservableCollection(Of CountUrlBytesViewModel)
    Public Property Operations() As ObservableCollection(Of CountUrlBytesViewModel)
        Get
            Return privateOperations
        End Get
        Private Set(ByVal value As ObservableCollection(Of CountUrlBytesViewModel))
            privateOperations = value
        End Set
    End Property

    Private privateCountUrlBytesCommand As ICommand
    Public Property CountUrlBytesCommand() As ICommand
        Get
            Return privateCountUrlBytesCommand
        End Get
        Private Set(ByVal value As ICommand)
            privateCountUrlBytesCommand = value
        End Set
    End Property

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Private Sub OnPropertyChanged(<CallerMemberName> Optional ByVal propertyName As String = Nothing)
        Dim handler As PropertyChangedEventHandler = PropertyChangedEvent
        If handler IsNot Nothing Then
            handler(Me, New PropertyChangedEventArgs(propertyName))
        End If
    End Sub
End Class
